import gym
import numpy
from numpy import random
from numba import njit
from l3c.anymdp.solver import update_value_matrix, get_final_transition, get_final_reward


class AnyMDPSolverOpt(object):
    """
    Solver for AnyMDPEnv with Bellman Equation and Value Iteration
    Suppose to know the ground truth of the environment
    """
    def __init__(self, env, gamma=0.98):
        if(not env.task_set):
            raise Exception("AnyMDPEnv is not initialized by 'set_task', must call set_task first")
        self.n_actions = env.action_space.n
        self.n_states = env.observation_space.n
        self.transition_matrix = get_final_transition(
                transition=env.transition_matrix,
                reset_states=env.reset_states,
                reset_triggers=env.reset_triggers)
        self.reward_matrix = get_final_reward(
                reward=env.reward_matrix,
                reset_triggers=env.reset_triggers,
        )
        self.state_mapping = env.state_mapping
        self.value_matrix = numpy.zeros((self.n_states, self.n_actions))
        self.gamma = gamma
        self.inverse_state_mapping = dict()
        for i,state in enumerate(self.state_mapping):
            self.inverse_state_mapping[state] = i
        self.q_solver()

    def q_solver(self, gamma=0.99):
        self.value_matrix = update_value_matrix(self.transition_matrix, self.reward_matrix, gamma, self.value_matrix)

    def policy(self, state):
        return numpy.argmax(self.value_matrix[self.inverse_state_mapping[state]])


"""
Gym Environment For Any MDP
"""
import numpy
import gym
import pygame
from numpy import random

from gym import error, spaces, utils
from gym.utils import seeding
from l3c.utils import pseudo_random_seed

class AnyMDPEnv(gym.Env):
    def __init__(self, max_steps):
        """
        Pay Attention max_steps might be reseted by task settings
        """
        self.observation_space = spaces.Discrete(2)
        self.action_space = spaces.Discrete(2)
        self.max_steps = max_steps
        self.task_set = False

    def set_task(self, task_config):
        self.transition_matrix = task_config["transition"]
        self.reward_matrix = task_config["reward"]
        self.reward_noise = task_config["reward_noise"]
        self.reward_noise_type = task_config["reward_noise_type"]
        self.state_mapping = task_config["state_mapping"]
        self.reset_triggers = task_config["reset_triggers"]
        self.reset_states = task_config["reset_states"]
        self.n_states = task_config["state_space"]
        self.n_actions = task_config["action_space"]

        if("max_steps" in task_config):
            self.max_steps = task_config["max_steps"]

        ns1, na1, ns2 = self.transition_matrix.shape
        ns3, na2, ns4 = self.reward_matrix.shape

        assert ns1 == ns2 and ns2 == ns3 and ns3==ns4, \
            "Transition matrix and reward matrix must have the same number of states"
        assert na1 == na2, \
            "Transition matrix and reward matrix must have the same number of actions"
        assert ns1 > 0, "State space must be at least 1"
        assert na1 > 1, "Action space must be at least 2"

        self.observation_space = spaces.Discrete(self.n_states)
        self.action_space = spaces.Discrete(self.n_actions)

        self.task_set = True
        self.need_reset = True

    def reset(self):
        if(not self.task_set):
            raise Exception("Must call \"set_task\" first")
        
        self.steps = 0
        self.need_reset = False
        random.seed(pseudo_random_seed())

        self._state = numpy.random.choice(len(self.reset_states),
                                          replace=True,
                                          p=self.reset_states)
        return self.state_mapping[self._state], {"steps": self.steps}

    def step(self, action):
        if(self.need_reset or not self.task_set):
            raise Exception("Must \"set_task\" and \"reset\" before doing any actions")
        assert action < self.n_actions, "Action must be less than the number of actions"
        transition_gt = self.transition_matrix[self._state, action]

        next_state = random.choice(len(self.state_mapping), p=transition_gt)

        reward_gt = self.reward_matrix[self._state, action, next_state]
        reward_gt_noise = self.reward_noise[self._state, action, next_state]

        if(self.reward_noise_type == 'normal'):
            reward = random.normal(reward_gt, reward_gt_noise)
        elif(self.reward_noise_type == 'binomial'):
            if(reward_gt > 0):
                reward = float(random.binomial(1, reward_gt))
            else:
                reward = - float(random.binomial(1, abs(reward_gt)))

        info = {"steps": self.steps, "reward_gt": reward_gt}

        if(self.state_mapping.ndim == 1):
            transition_ext_gt = numpy.zeros((self.n_states,))
            for i,s in enumerate(self.state_mapping):
                transition_ext_gt[s] = transition_gt[i]
            info["transition_gt"] = transition_ext_gt

        self.steps += 1
        self._state = next_state
        #print("inner", next_state, "outer", self.state_mapping[next_state], self.max_steps,
        #      "triggers", self.reset_triggers, "starts", self.reset_states)
        done = (self.steps >= self.max_steps or self.reset_triggers[self._state])
        if(done):
            self.need_reset = True
        return self.state_mapping[next_state], reward, done, info
    
    @property
    def state(self):
        return self.state_mapping[self._state]
    
    @property
    def inner_state(self):
        return self._state

class AnyMDPEnvD2C(AnyMDPEnv):
    """
    Transfer a AnyMDPEnv to Continuous State Space without resampling a task
    """
    def __init__(self, max_steps, state_dim):
        super().__init__(max_steps)
        self.observation_space = spaces.Box(0., 1., shape=(state_dim,))
        self.state_dim = state_dim

    def set_task(self, task_config):
        super().set_task(task_config)
        n_inner_states = self.state_mapping.shape[0]
        self.state_mapping = numpy.random.normal(size=(n_inner_states, self.state_dim))
        self.observation_space = spaces.Box(0., 1., shape=(self.state_dim,))


import gym
import numpy
from numpy import random
from numba import njit
import networkx as nx
import scipy.stats as stats

@njit(cache=True)
def update_value_matrix(t_mat, r_mat, gamma, vm, max_iteration=-1, is_greedy=True):
    diff = 1.0
    cur_vm = numpy.copy(vm)
    ns, na, _ = r_mat.shape
    iteration = 0
    while diff > 1.0e-4 and (
            (max_iteration < 0) or 
            (max_iteration > iteration and max_iteration > 1) or
            (iteration < 1 and random.random() < max_iteration)):
        iteration += 1
        old_vm = numpy.copy(cur_vm)
        for s in range(ns):
            for a in range(na):
                exp_q = 0.0
                for sn in range(ns):
                    if(is_greedy):
                        exp_q += t_mat[s,a,sn] * numpy.max(cur_vm[sn])
                    else:
                        exp_q += t_mat[s,a,sn] * numpy.mean(cur_vm[sn])
                cur_vm[s,a] = numpy.dot(r_mat[s,a], t_mat[s,a]) + gamma * exp_q
        diff = numpy.sqrt(numpy.mean((old_vm - cur_vm)**2))
    return cur_vm

def get_final_transition(**task):
    t_mat = numpy.copy(task["transition"])
    if(t_mat.shape[0] < 2):
        return t_mat
    
    reset_dist = task["reset_states"]
    reset_trigger = numpy.where(task["reset_triggers"] > 0)

    for s in reset_trigger:
        t_mat[s, :] = reset_dist

    return t_mat

def get_final_reward(**task):
    r_mat = numpy.copy(task["reward"])
    if(r_mat.shape[0] < 2):
        return r_mat
    reset_trigger = numpy.where(task["reset_triggers"] > 0)

    r_mat[reset_trigger, :, :] = 0.0

    return r_mat

def check_transition(t_mat):
    quality = -1
    if(t_mat is None):
        return quality
    # acquire state - to - state distribution
    ns = t_mat.shape[0]
    log_ns = int(numpy.floor(numpy.log2(ns)))
    ss_trans = numpy.sum(t_mat, axis=1)
    ss_trans = ss_trans / numpy.sum(ss_trans, axis=1, keepdims=True)

    for i in range(log_ns):
        ss_trans = numpy.matmul(ss_trans, ss_trans)
        ss_unreach = numpy.sum(ss_trans < 1.0e-6)
        if(ss_unreach > 0):
            quality = max(quality, i / log_ns + ss_unreach / ns / ns)
    ss_unreach = numpy.sum(ss_trans < 1.0e-6, axis=1)
    if(numpy.any(ss_unreach > 0)):
        return 0
    if(ns < 4):
        return 1 # where states below 4, transition is all ok as long as strongly connected
    return quality

def check_valuefunction(t_mat, r_mat):
    if(t_mat is None or r_mat is None):
        return -100
    ns, na, _ = r_mat.shape
    if(ns < 2): # For bandit problem, only check rewards
        if(numpy.std(r_mat) > 1.0e-3):
            return 1
        else:
            return -100

    vm_l = update_value_matrix(t_mat, r_mat, 0.99, numpy.zeros((ns, na), dtype=float), max_iteration=5)
    vm_s = update_value_matrix(t_mat, r_mat, 0.70, numpy.zeros((ns, na), dtype=float), max_iteration=5)
    vm_r = update_value_matrix(t_mat, r_mat, 0.99, numpy.zeros((ns, na), dtype=float), max_iteration=5, is_greedy=False)

    vm_l = numpy.max(vm_l, axis=1)
    vm_s = numpy.max(vm_s, axis=1)
    vm_r = numpy.max(vm_r, axis=1)

    vbase = numpy.sqrt(numpy.mean(vm_r ** 2))

    corr_ls, _ = stats.spearmanr(vm_l, vm_s)
    corr_lr, _ = stats.spearmanr(vm_l, vm_r)
    qdelta = numpy.mean((vm_l - vm_r)) / vbase
    qstd = numpy.std(vm_l) / vbase

    if(qstd < 0.1): # value function too flat
        return -100

    if(numpy.isnan(corr_lr)):
        corr_lr = 1
    if(numpy.isnan(corr_ls)):
        corr_ls = 1

    quality = qstd + qdelta +  numpy.log(1 + 1.0e-8 - corr_lr) + numpy.log(1 + 1.0e-8 - corr_ls)

    return quality


def check_task_trans(task):
    """
    Check the quality of the task
    Requiring: Q value is diverse
               State is connected
               Route is complex
    Returns:
        float: transition quality
        float: value function quality
    """
    if(task is None or 
       not "transition" in task or 
       not "reset_states" in task or
       not "reset_triggers" in task):
        return -1
    if(task["transition"].shape[0] < 2):
        return 1
    t_mat = get_final_transition(**task)
    return check_transition(t_mat)


def check_task_rewards(task):
    """
    Check the quality of the task
    Requiring: Q value is diverse
               State is connected
               Route is complex
    Returns:
        float: transition quality
        float: value function quality
    """
    if(task is None or 
       not "transition" in task or 
       not "reset_states" in task or
       not "reset_triggers" in task or 
       not "reward" in task):
        return -100

    t_mat = get_final_transition(**task)
    r_mat = get_final_reward(**task)
    return check_valuefunction(t_mat, r_mat)


#   Copyright (c) 2021 DeepEvolution Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import numpy
import gym
from numpy import random


class RandomNGram(object):
    def __init__(self, task):
        for key, val in task.items():
            self.__dict__[key] = val
        self.s_tok = 0
        self.w_arr = numpy.expand_dims(numpy.arange(self.n_gram), axis=[0, 2, 3])

    def softmax(self, x):
        e_x = numpy.exp(x - numpy.max(x, axis=-1, keepdims=True))
        return e_x / e_x.sum(axis=-1, keepdims=True)

    def forward(self, l, batch=1, seed=None):
        #Generate n=batch sequences of length l under current task
        ind = 0
        if(seed is not None):
            numpy.random.seed(seed)

        def mean_var_norm(i):
            m_i = numpy.mean(i)
            m_ii = numpy.mean(i * i)
            std = numpy.sqrt(m_ii - m_i * m_i)
            return (1.0 / std) * (i - m_i)

        cur_tok = numpy.full((batch,), self.s_tok)
        idxes = numpy.arange(batch)
        pad_emb = numpy.expand_dims(self.word_emb[cur_tok], axis=1)
 
        h = numpy.zeros((batch, self.n_hidden))

        # mark whether there is end token
        seqs = []
        seqs.append(cur_tok)
        ppl = 0
        tok_cnt = 0
        tok_embs = [pad_emb for _ in range(self.n_gram)]
        while ind < l:
            ind += 1
            tok_emb = numpy.expand_dims(self.word_emb[cur_tok], axis=1)
            tok_embs.append(tok_emb)
            del tok_embs[0]
            tok_emb = numpy.expand_dims(numpy.concatenate(tok_embs[-self.n_gram:], axis=1), axis=2)

            h = numpy.tanh(numpy.matmul(tok_emb, self.weights_inputlayer) + self.bias_inputlayer)
            h = numpy.mean(self.w_arr * h, axis=1)
            o = numpy.matmul(h, self.weights_outputlayer) + self.bias_outputlayer
            o = numpy.squeeze(o, axis=1)
            o = self._lambda * mean_var_norm(o)
            exp_o = numpy.exp(o)
            prob = exp_o / numpy.sum(exp_o, axis=-1, keepdims=True)
            cur_tok = (prob.cumsum(1) > numpy.random.rand(prob.shape[0])[:,None]).argmax(1)
            cur_prob = prob[idxes, cur_tok]
            ppl -= numpy.sum(numpy.log(cur_prob))
            tok_cnt += cur_prob.shape[0]

            seqs.append(cur_tok)
        print("Ground Truth Sequence Perplexity: %f" % (ppl / tok_cnt))

        return numpy.transpose(numpy.asarray(seqs, dtype="int32"))

class MetaLangV2(gym.Env):
    """
    Pseudo Langauge Generated from RNN models
    V: vocabulary size
    d: embedding size (input size)
    n: n-gram
    N: hidden size
    e: inverse of softmax - temporature
    L: maximum length
    """
    def __init__(self, L=4096):
        self.L = int(L)
        assert L>1
        self.task_set = False

    def set_task(self, task):
        self.nn = RandomNGram(task)
        self.task_set = True

    def data_generator(self, seed=None):
        if(self.task_set):
            tokens = self.nn.forward(self.L, seed=seed)[0]
        else:
            raise Exception("Please set task before using data generator")
        return tokens

    def batch_generator(self, batch_size, seed=None):
        if(self.task_set):
            tokens = self.nn.forward(self.L, batch=batch_size, seed=seed)
        else:
            raise Exception("Please set task before using data generator")
        return tokens




import gym
import numpy
from numpy import random

class AnyMDPSolverQ(object):
    """
    Solver for AnyMDPEnv with Q-Learning
    """
    def __init__(self, env, gamma=0.99, c=0.01, alpha=0.01, max_steps=4000):
        """
        The constructor for the class AnyMDPSolverQ
        The exploration strategy is controlled by UCB-H with c as its hyperparameter. Increasing c will encourage exploration
        Simulation of the ideal policy when the ground truth is not known
        """
        if(not env.task_set):
            raise Exception("AnyMDPEnv is not initialized by 'set_task', must call set_task first")
        self.n_actions = env.action_space.n
        self.n_states = env.observation_space.n
        self.transition_matrix = env.transition_matrix
        self.reward_matrix = env.reward_matrix
        self.value_matrix = numpy.zeros((self.n_states, self.n_actions))
        self.sa_vistied = numpy.zeros((self.n_states, self.n_actions))
        self.gamma = gamma
        self.alpha = alpha
        self.max_steps = max_steps
        self._c = c / (1.0 - self.gamma)


    def learner(self, s, a, ns, r, done):
        if(done):
            target = r
        else:
            target = r + self.gamma * max(self.value_matrix[ns])
        error = target - self.value_matrix[s][a]
        self.value_matrix[s][a] += self.alpha * error
        self.sa_vistied[s][a] += 1

    def policy(self, state):
        # Apply UCB with dynamic noise (Thompson Sampling)
        values = self._c * numpy.sqrt(numpy.log(self.max_steps + 1) / numpy.clip(self.sa_vistied[state], 1.0, None)) * \
                numpy.maximum(numpy.random.randn(self.n_actions), 0) + \
                self.value_matrix[state]
        return numpy.argmax(values)


#!/usr/bin/env python
# coding=utf8
# File: test.py
import gym
import sys
import l3c.mazeworld
from l3c.mazeworld import MazeTaskSampler
from numpy import random

def test_maze(max_steps=1000):
    maze_env = gym.make("mazeworld-v2", enable_render=False, max_steps=max_steps)
    task = MazeTaskSampler(verbose=True)
    maze_env.set_task(task)

    maze_env.reset()
    done=False
    sum_reward = 0
    while not done:
        state, reward, done, _ = maze_env.step(maze_env.action_space.sample())
        sum_reward += reward
    print("...Test Finishes. Get score %f, for maze_type = %s task_type = %s, n = %d, steps = %s\n\n---------\n\n"%(sum_reward, maze_type, task_type, n, max_steps))

if __name__=="__main__":
    for n in [9, 15, 25]:
        for task_type in ["NAVIGATION", "SURVIVAL"]:
            for maze_type in ["Discrete2D", "Discrete3D", "Continuous3D"]:
                n_landmarks=random.randint(2,10)
                density=random.random() * 0.50
                test_maze(n=n, task_type=task_type, density=density, maze_type=maze_type, n_landmarks=n_landmarks)
    print("\n\nCongratulations!!!\n\nAll Tests Have Been Passed\n\n")



"""
Gym Environment For Maze3D
"""
import numpy
import gym
import pygame

from gym import error, spaces, utils
from gym.utils import seeding
from l3c.mazeworld.envs.maze_continuous_3d import MazeCoreContinuous3D
from l3c.mazeworld.envs.dynamics import DEFAULT_ACTION_SPACE_16, DEFAULT_ACTION_SPACE_32

class MazeWorldEnvBase(gym.Env):
    """
    All Maze World Environments Use This Base Class
    """
    def __init__(self, 
            maze_type,
            enable_render=True,
            render_scale=480,
            max_steps=5000,
            ):
        self.maze_type = maze_type
        self.enable_render = enable_render
        self.render_viewsize = render_scale

        self.need_reset = True
        self.need_set_task = True

    def set_task(self, task_config):
        self.maze_core.set_task(task_config)
        self.need_set_task = False

    def reset(self):
        if(self.need_set_task):
            raise Exception("Must call \"set_task\" before reset")
        state = self.maze_core.reset()
        if(self.enable_render):
            self.maze_core.render_init(self.render_viewsize)
            self.keyboard_press = pygame.key.get_pressed()
        info = {"steps": self.maze_core.steps}
        self.maze_core.get_info(info)
        self.need_reset = False
        self.key_done = False
        return state, info

    def action_control(self, action):
        raise NotImplementedError("Must implement the action control logic")

    def step(self, action=None):
        if(self.need_reset):
            raise Exception("Must \"reset\" before doing any actions")

        internal_action = self.action_control(action)
            
        # In keyboard control, process only continues when key is pressed
        info = {"steps": self.maze_core.steps}
        if(internal_action is None):
            return self.maze_core.get_observation(), 0, False, info 
        reward, done = self.maze_core.do_action(internal_action)
        self.maze_core.get_info(info)

        if(done):
            self.need_reset=True

        return self.maze_core.get_observation(), reward, done, info

    def render(self, mode="human"):
        if(mode != "human"):
            raise NotImplementedError("Only human mode is supported")
        if(self.enable_render):
            self.key_done, self.keyboard_press = self.maze_core.render_update()

    def get_local_map(self, map_range=8, resolution=(128, 128)):
        """
        Returns the localized god-view map relative to the agent's standpoint
        """
        return self.maze_core.get_local_map(map_range=map_range, resolution=resolution)

    def get_global_map(self, resolution=(128, 128)):
        """
        Returns the global god-view map
        """
        return self.maze_core.get_global_map(resolution=resolution)

    def get_target_location(self):
        """
        Acquire relative position of the target to the agent
        Return (Distance, Angle)
        """
        target_id = self.maze_core._commands_sequence[self.maze_core._commands_sequence_idx]
        target_grid = self.maze_core._landmarks_coordinates[target_id]
        deta_grid = numpy.zeros(shape=(2,), dtype=numpy.float32)
        deta_grid[0] = target_grid[0] - self.maze_core._agent_grid[0]
        deta_grid[1] = target_grid[1] - self.maze_core._agent_grid[1]
        angle = numpy.arctan2(deta_grid[1], deta_grid[0]) - self.maze_core._agent_ori
        if(angle < -numpy.pi):
            angle += 2 * numpy.pi
        elif(angle > numpy.pi):
            angle -= 2 * numpy.pi
        dist = numpy.sqrt(numpy.sum(deta_grid * deta_grid))
        return dist, angle

    def save_trajectory(self, file_name, view_size=480):
        if(not self.enable_render):
            self.maze_core.render_init(view_size)
        self.maze_core.render_trajectory(file_name)

class MazeWorldContinuous3D(MazeWorldEnvBase):
    def __init__(self, 
            enable_render=True,
            render_scale=480,
            max_steps = 5000,
            resolution=(320, 320),
            visibility_3D=12.0,
            command_in_observation=False,
            action_space_type="Discrete16",  # Must choose in Discrete16, Discrete32, Continuous
            ):
        super(MazeWorldContinuous3D, self).__init__(
            "Continuous3D",
            enable_render=enable_render,
            render_scale=render_scale,
            max_steps=max_steps
        )
        self.maze_core = MazeCoreContinuous3D(
                resolution_horizon = resolution[0],
                resolution_vertical = resolution[1],
                max_steps = max_steps,
                visibility_3D=visibility_3D,
                command_in_observation=command_in_observation
                )
        
        self.inner_action_list  = None
        if(action_space_type == "Discrete16"):
            self.action_space = spaces.Discrete(16)
            # Using Default Discrete Actions
            self.inner_action_list = DEFAULT_ACTION_SPACE_16
        elif(action_space_type == "Discrete32"):
            self.action_space = spaces.Discrete(32)
            # Using Default Discrete Actions
            self.inner_action_list = DEFAULT_ACTION_SPACE_32
        elif(action_space_type == "Continuous"):
            # Turning Left/Right and go backward / forward
            self.action_space = spaces.Box(low=-1, high=1, shape=(2,), dtype=numpy.float32)
        else:
            raise ValueError("Invalid Action Space Type {}. Can only accept Discrete16, Discrete32, Continuous".format(action_space_type))

        # observation is the x, y coordinate of the grid
        self.observation_space = spaces.Box(low=0, high=255, shape=(resolution[0], resolution[1], 3), dtype=numpy.uint8)

    def action_control(self, action):
        if(action is None): # Only when there is no action input can we use keyboard control
            pygame.time.delay(20) # 50 FPS
            tr, ws = self.maze_core.movement_control(self.keyboard_press)
        else:
            if(self.inner_action_list is not None):
                tr, ws = self.inner_action_list[action]
            else:
                tr, ws = action
        if(tr is None or ws is None):
            return None
        return [tr, ws]
    
    @property
    def list_actions(self):
        return self.inner_action_list


from l3c.mazeworld.envs.maze_env import MazeWorldContinuous3D
from l3c.mazeworld.envs.task_sampler import MAZE_TASK_MANAGER, MazeTaskSampler, Resampler
from l3c.mazeworld.envs.grid_ops import genmaze_by_primwall
from l3c.mazeworld.envs.utils import conv2d_numpy


#   Copyright (c) 2022 DeepEvolution Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import io
from setuptools import setup, find_packages

__version__ = '0.2.1.17'

with io.open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

setup(
    name='l3c',
    version=__version__,
    author='WorldEditors',
    author_email='',
    description=('Benchmarks for Lifelong In-Context Learning'),
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/FutureAGI/L3C',
    license="Apache",
    packages=[package for package in find_packages()
              if package.startswith('l3c')],
    package_data={'l3c': [
        './mazeworld/envs/img/*',
        ]
    },
    python_requires='>=3.7',
    tests_require=['pytest', 'mock'],
    include_package_data=True,
    install_requires=[
        'gym>=0.18.0',
        'numpy>=1.24.4',
        'Pillow>=6.2.2',
        'six>=1.12.0',
        'pygame>=2.6.0',
        'numba>=0.58.1'
    ],
    extras_require={},
    zip_safe=False,
)



#   Copyright (c) 2018 PaddlePaddle Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from gym.envs.registration import register
from l3c.metalang.metalangv1 import MetaLangV1
from l3c.metalang.metalangv2 import MetaLangV2
from l3c.metalang.task_sampler import TaskSamplerV1, TaskSamplerV2
from l3c.metalang.generator import metalang_generator

register(
    id='meta-language-v1',
    entry_point='l3c.metalang:MetaLangV1',
    kwargs={"L": 2048}
)

register(
    id='meta-language-v2',
    entry_point='l3c.metalang:MetaLangV2',
    kwargs={"L": 2048}
)
